from django.db import models
from django.contrib.auth.models import User as DjangoUser, check_password
from django.contrib.auth import authenticate, login
from django.shortcuts import render_to_response
from django.conf import settings
from django.http import Http404
from surftrackr.log.models import ViewPermissions, UserProfile

def http_login(request):
    if settings.DISABLE_AUTH:
        rem_user = 'surftrackr'
    else:
        rem_user = request.META['REMOTE_USER']

    if (rem_user == None): raise Http404

    try:
        django_user = DjangoUser.objects.get(username=rem_user)
    except DjangoUser.DoesNotExist:
        pw = DjangoUser.objects.make_random_password(length=10, allowed_chars='abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ23456789')
        django_user = DjangoUser.objects.create_user(username=rem_user, email=rem_user + '@example.com', password=pw)
        django_user.is_staff = True
        django_user.is_superuser = True
        view_perms = ViewPermissions.objects.all()
        user_profile, created = UserProfile.objects.get_or_create(user=django_user)
        for vp in view_perms: user_profile.view_permissions.add(vp)
        django_user.save()

    if django_user.is_active:
        django_user.backend='django.contrib.auth.backends.ModelBackend' 
        login(request, django_user)
        return django_user
    else:
        raise Http404

def account_not_found(request):
    http_auth_name = request.META['REMOTE_USER']
    return render_to_response('httpauth/locked.html', locals())

def check_login(request):
    user = http_login(request)
    if not user: return account_not_found(request)

def check_is_staff(request):
    user = http_login(request)
    if not user.is_staff: return account_not_found(request)

def get_view_permissions(duser_obj):
    p = duser_obj.get_profile()
    return dict([(i['name'],True) for i in p.view_permissions.values()])



