import datetime, sets, os
from django import newforms as forms
from django.newforms import *
from django.template import Context, loader
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import get_object_or_404, render_to_response
from surftrackr.log.models import *
from surftrackr.httpauth.models import *
from surftrackr.preferences.models import *
from surftrackr.wordsearch.models import *
from django.contrib.admin.models import LogEntry
from django.conf import settings
from django.db import connection
from django.contrib.auth.models import User as DjangoUser
from django.newforms.util import ErrorList
from dateutil.parser import *
import datetime


DAYS = [(1,'1 day'), (2,'2 days'), (3,'3 days'), (4,'4 days'), (5,'5 days'), (6,'6 days'), (7,'7 days'), (14,'14 days'), (21,'21 days'), (28,'28 days')]


def page_not_found(request, template_name='404.html'): 
    return render_to_response(template_name, locals())


def get_general_setting(request):
    duser_obj = http_login(request)

    try:
        general_setting = GeneralSetting.objects.get(owner=duser_obj)
    except GeneralSetting.DoesNotExist:
        general_setting = GeneralSetting.objects.create(start_date=datetime.datetime.now() - datetime.timedelta(hours=24), end_date=datetime.datetime.now(), row_limit=0, owner=duser_obj, use_thumbnails=False)

    return general_setting


def pref_days_ago(request):
    general_setting = get_general_setting(request)
    n = datetime.datetime.now()
    d = datetime.timedelta(days=1)

    return n - d


def pref_start_date(request):
    general_setting = get_general_setting(request)
    return general_setting.start_date


def pref_end_date(request):
    general_setting = get_general_setting(request)
    return general_setting.end_date


def pref_row_limit(request):
    duser_obj = http_login(request)

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs = Weblog.objects.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
    else:
        weblogs = Weblog.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

    row_limit = get_general_setting(request).row_limit
    if (row_limit == 0): row_limit = 10000

    return row_limit


def pref_use_thumbnails(request):
    use_thumbnails = get_general_setting(request).use_thumbnails
    return use_thumbnails


def front_page(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    message = 'Viewing front page.'

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            weblogs = Weblog.objects.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
        elif 'Websites' in view_permissions:
            weblogs = Weblog.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
    except Weblog.DoesNotExist:
        weblogs = 0

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            users = User.objects.filter(exclude_from_reports=False).count()
        elif 'Users' in view_permissions:
            users = User.objects.filter(managed_by=duser_obj).filter(exclude_from_reports=False).count()
    except User.DoesNotExist:
        users = 0

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            workstations = Workstation.objects.filter(user__exclude_from_reports=False).count()
        elif 'Workstations' in view_permissions:
            workstations = Workstation.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).count()
    except Workstation.DoesNotExist:
        workstations = 0

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH or ('Lexicons' in view_permissions)):
            lexicons = Lexicon.objects.all().count()
    except Lexicon.DoesNotExist:
        lexicons = 0

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH or ('HiLites' in view_permissions)):
            hilites = HiLite.objects.all().count()
    except HiLite.DoesNotExist:
        hilites = 0

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH or ('Websites' in view_permissions)):
            websites = Website.objects.filter(last_visit__gt=pref_start_date(request)).filter(last_visit__lt=pref_end_date(request)).count()
    except Website.DoesNotExist:
        websites = 0

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            flagged_weblogs = Weblog.objects.filter(user__exclude_from_reports=False).filter(flagged=True).count()
        elif 'Flagged' in view_permissions:
            flagged_weblogs = Weblog.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(flagged=True).count()
    except Weblog.DoesNotExist:
        flagged_weblogs = 0

    try:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            flagged_users = User.objects.filter(exclude_from_reports=False).filter(flagged=True).count()
        elif 'Flagged' in view_permissions:
            flagged_users = User.objects.filter(managed_by=duser_obj).filter(exclude_from_reports=False).filter(flagged=True).count()
    except User.DoesNotExist:
        flagged_users = 0

    row_limit = pref_row_limit(request)

    class FreeFormDateField(forms.Field):
        def clean(self, value):
            if not value: raise forms.ValidationError('Please enter a valid date and time.')
            try:
                if settings.US_DATE_FORMAT:
                    the_date = parse(value, dayfirst=False)
                else:
                    the_date = parse(value, dayfirst=True)
            except ValueError:
                raise forms.ValidationError('Date format not recognized.')
            return the_date

    class RangeForm(forms.Form):
        start_date = FreeFormDateField(widget=TextInput(attrs={'size':20}))
        end_date   = FreeFormDateField(widget=TextInput(attrs={'size':20}))

        def save(self):
            if self.is_valid():
                general_setting = GeneralSetting.objects.get(owner=duser_obj)
                general_setting.start_date = self.cleaned_data['start_date']
                general_setting.end_date = self.cleaned_data['end_date']
                if general_setting.start_date < general_setting.end_date:
                    general_setting.save()

    if request.method == 'POST':
        if not settings.DEMO_MODE:
            range_form = RangeForm(request.POST)
            if range_form.is_valid():
                range_form.save()
                return HttpResponseRedirect('/')
    else:
        range_form = RangeForm({ 'start_date': get_general_setting(request).start_date, 'end_date': get_general_setting(request).end_date })

    if settings.DEMO_MODE: demo_mode = True
    if not settings.DISABLE_AUTH: show_auth = True


    time_map = { 'm':'1', 'h':'2', 'd':'3', 'w':'4'  }
    time_buttons = [(str(t.number), t.units, t) for t in TimeSlice.objects.all()]
    time_buttons.sort(lambda x, y: cmp(time_map[x[1]] + "%010.0f" % int(x[0]), time_map[y[1]] + "%010.0f" % int(y[0])))


    return render_to_response('frontpage.html', locals())


def count_websites_for_user(request, user_obj):
    start_date = pref_start_date(request).strftime("%Y-%m-%d %H:%M:%S")
    end_date   = pref_end_date(request).strftime("%Y-%m-%d %H:%M:%S")

    cursor = connection.cursor()
    sql = \
        "SELECT COUNT(DISTINCT(website_id)) FROM log_weblog WHERE user_id=" + str(user_obj.id) + \
        " AND log_weblog.log_datetime >= '" + start_date + "' AND log_weblog.log_datetime <= '" + end_date + "'"

    cursor.execute(sql)

    if settings.DEBUG:
        count_tuple = cursor.cursor.fetchall()
    else:
        count_tuple = cursor.fetchall()

    return count_tuple[0][0]


def show_users(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Users' in view_permissions)): raise Http404

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        users_obj = User.objects.filter(exclude_from_reports=False)[:pref_row_limit(request)]
    else:
        users_obj = User.objects.filter(managed_by=duser_obj).filter(exclude_from_reports=False)[:pref_row_limit(request)]

    for u in users_obj:
        u.weblog_matches = u.weblog_set.filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
        u.website_matches = count_websites_for_user(request, u)
        u.image_matches = u.weblog_set.filter(mimetype__type='image').filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

    message = '<a href="/">Main Page</a> &rarr; Users'
    return render_to_response('users/list.html', locals())


def show_user_images(request, user_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    user_obj = get_object_or_404(User, pk=user_id)

    if not (duser_obj.is_superuser or settings.DEMO_MODE):
        if not user_obj.managed_by == duser_obj: raise Http404

    weblogs_obj = user_obj.weblog_set.filter(mimetype__type='image').filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/users/show/">Users</a> &rarr; Images for <em>' + str(user_obj) + '</em>'
    return render_to_response('users/list_images.html', locals())


def get_sitenames_for_user(request, user_obj):
    start_date = pref_start_date(request).strftime("%Y-%m-%d %H:%M:%S")
    end_date   = pref_end_date(request).strftime("%Y-%m-%d %H:%M:%S")

    cursor = connection.cursor()
    sql = \
        "SELECT DISTINCT(log_website.id), log_website.name FROM log_website,log_weblog WHERE log_website.id=log_weblog.website_id " + \
        "AND log_weblog.user_id=" + str(user_obj.id) + \
        " AND log_weblog.log_datetime >= '" + start_date + "' AND log_weblog.log_datetime <= '" + end_date + \
        "' ORDER BY log_website.name"

    if (pref_row_limit(request) > 0): sql = sql + " LIMIT " + str(pref_row_limit(request))

    cursor.execute(sql)

    if settings.DEBUG:
        return cursor.cursor.fetchall()
    else:
        return cursor.fetchall()


def show_urls_for_user(request, user_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    user_obj = get_object_or_404(User, pk=user_id)

    if not (duser_obj.is_superuser or settings.DEMO_MODE):
        if not user_obj.managed_by == duser_obj: raise Http404

    weblogs_obj = user_obj.weblog_set.filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)] 

    message = '<a href="/">Main Page</a> &rarr; <a href="/users/show/">Users</a> &rarr; URLs for ' + str(user_obj)
    return render_to_response('users/urls_for_user.html', locals())


def show_user_websites(request, user_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    user_obj = get_object_or_404(User, pk=user_id)
    if not (duser_obj.is_superuser or settings.DEMO_MODE):
        if not user_obj.managed_by == duser_obj: raise Http404

    websites_list = get_sitenames_for_user(request, user_obj)[:pref_row_limit(request)]
    websites_obj = []
    for w in websites_list: websites_obj.append(Website.objects.get(pk=int(w[0])))
    use_thumbnails = pref_use_thumbnails(request)

    message = '<a href="/">Main Page</a> &rarr; <a href="/users/show/">Users</a> &rarr; Websites for ' + str(user_obj)
    return render_to_response('users/user.html', locals())


def show_workstations(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Workstations' in view_permissions)): raise Http404

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        workstations_obj = Workstation.objects.filter(user__exclude_from_reports=False)[:pref_row_limit(request)]
    else:
        workstations_obj = Workstation.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False)[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; Workstations'
    return render_to_response('workstations/list.html', locals())


def show_workstation(request, workstation_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Workstations' in view_permissions)): raise Http404

    workstation_obj = get_object_or_404(Workstation, pk=workstation_id)

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        workstation_users_obj = workstation_obj.user_set.filter(exclude_from_reports=False)[:pref_row_limit(request)]
    else:
        workstation_users_obj = workstation_obj.user_set.filter(managed_by=duser_obj).filter(exclude_from_reports=False)[:pref_row_limit(request)]

    for w in workstation_users_obj:
        w.weblog_matches = w.weblog_set.filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
        w.website_matches = count_websites_for_user(request, w)

    message = '<a href="/">Main Page</a> &rarr; <a href="/workstations/show/">Workstations</a> &rarr; ' + str(workstation_obj) + ' &rarr; Users'
    return render_to_response('workstations/workstation.html', locals())


def show_websites(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Websites' in view_permissions)): raise Http404

    websites_obj = Website.objects.filter(last_visit__gt=pref_start_date(request)).filter(last_visit__lt=pref_end_date(request))[:pref_row_limit(request)]
    use_thumbnails = pref_use_thumbnails(request)

    for w in websites_obj:
        w.images = w.weblog_set.filter(mimetype__type='image').filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

    message = '<a href="/">Main Page</a> &rarr; Websites'
    return render_to_response('websites/list.html', locals())


def show_website(request, website_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    website_obj = get_object_or_404(Website, pk=website_id)
    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs_obj = website_obj.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]
    else:
        weblogs_obj = website_obj.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]

    if website_obj.name == website_obj.top_level_site.name:
        child_sites_obj = website_obj.website_set.exclude(name=website_obj.name)[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/websites/show/">Websites</a> &rarr; ' + str(website_obj)
    return render_to_response('websites/urls.html', locals())


def show_website_graphics(request, website_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    website_obj = get_object_or_404(Website, pk=website_id)
    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs_obj = website_obj.weblog_set.filter(mimetype__type='image').filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]
    else:
        weblogs_obj = website_obj.weblog_set.filter(mimetype__type='image').filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/websites/show/">Websites</a> &rarr; Graphics for website: ' + str(website_obj)
    return render_to_response('websites/list_images.html', locals())


def show_visits_to_url(request, latest_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    weblog_obj = get_object_or_404(Weblog, pk=latest_id)

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs_obj = Weblog.objects.filter(user__exclude_from_reports=False).filter(url=weblog_obj.url)[:pref_row_limit(request)]
    else:
        weblogs_obj = Weblog.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(url=weblog_obj.url)[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/websites/show/">Websites</a> &rarr; ' + str(weblog_obj.website)
    return render_to_response('websites/url_visits.html', locals())


def show_urls_for_site_and_user(request, website, user_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    user_obj = get_object_or_404(User, pk=user_id)

    if not (duser_obj.is_superuser or settings.DEMO_MODE):
        if not user_obj.managed_by == duser_obj: raise Http404

    website_obj = get_object_or_404(Website, name=website)
    urls_obj = website_obj.weblog_set.filter(user=user_obj).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/users/show/">Users</a> &rarr; <a href="/users/show/' \
        + str(user_obj.id) + '/">' + str(user_obj) + '</a> &rarr; ' + str(website_obj)

    return render_to_response('urls/urls_for_website_and_user.html', locals())


def show_urls_for_site(request, website):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    website_obj = get_object_or_404(Website, name=website)

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        urls_obj = website_obj.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]
    else:
        urls_obj = website_obj.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]

    if website_obj.name == website_obj.top_level_site.name:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            child_sites_obj = website_obj.website_set.filter(weblog__user__exclude_from_reports=False).exclude(name=website_obj.name)[:pref_row_limit(request)]
        else:
            child_sites_obj = website_obj.website_set.filter(user__managed_by=duser_obj).filter(weblog__user__exclude_from_reports=False).exclude(name=website_obj.name)[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/websites/show/">Websites</a> &rarr; ' + str(website_obj)
    return render_to_response('urls/urls_for_website.html', locals())


def count_words_in_lexicon(request, lexicon):
    count_tuple = ()
    cursor = connection.cursor()
    sql = "SELECT COUNT(*) FROM log_weblog WHERE url LIKE '%%" + "%%' OR url LIKE '%%".join([w.text for w in lexicon.word.all()]) + "%%'"
    cursor.execute(sql)

    if settings.DEBUG:
        count_tuple = cursor.cursor.fetchall()
    else:
        count_tuple = cursor.fetchall()

    return count_tuple[0][0]


def show_lexicons(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Lexicons' in view_permissions)): raise Http404

    lexicons_obj = Lexicon.objects.all()
    for l in lexicons_obj: l.matches = count_words_in_lexicon(request, l)

    message = '<a href="/">Main Page</a> &rarr; Lexicons'
    return render_to_response('lexicons/list.html', locals())


def show_lexicon(request, lexicon_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Lexicons' in view_permissions)): raise Http404

    lexicon_obj = get_object_or_404(Lexicon, pk=lexicon_id)
    words_obj = lexicon_obj.word.all()

    for w in words_obj:
        w.matches = 0
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            w.matches += Weblog.objects.filter(user__exclude_from_reports=False).filter(url__icontains=w).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
        else:
            w.matches += Weblog.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(url__icontains=w).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

    message = '<a href="/">Main Page</a> &rarr; <a href="/lexicons/show/">Lexicons</a> &rarr; ' + str(lexicon_obj)
    return render_to_response('lexicons/words.html', locals())


def show_urls_for_word(request, word_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    word_obj = get_object_or_404(Word, pk=word_id)

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs_obj = Weblog.objects.filter(user__exclude_from_reports=False).filter(url__icontains=word_obj.text).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]
    else:
        weblogs_obj = Weblog.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(url__icontains=word_obj.text).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/lexicons/show/">Lexicons</a> &rarr; Word: ' + str(word_obj)
    return render_to_response('lexicons/urls_for_word.html', locals())


def show_hilites(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('HiLites' in view_permissions)): raise Http404

    hilites_obj = HiLite.objects.all()
    for h in hilites_obj: h.matches = h.website_set.filter(last_visit__gt=pref_start_date(request)).filter(last_visit__lt=pref_end_date(request)).count()

    message = '<a href="/">Main Page</a> &rarr; HiLites'
    return render_to_response('hilites/list.html', locals())


def show_hilite(request, hilite_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    hilite_obj = get_object_or_404(HiLite, pk=hilite_id)
    websites_obj = Website.objects.filter(hilite=hilite_obj).filter(last_visit__gt=pref_start_date(request)).filter(last_visit__lt=pref_end_date(request))

    for w in websites_obj:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            w.matches = w.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
        else:
            w.matches = w.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

        w.children = []
        for child_w in Website.objects.filter(top_level_site=w):
            if w == child_w: continue
            if (duser_obj.is_superuser or settings.DISABLE_AUTH):
                child_w.matches = child_w.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
            else:
                child_w.matches = child_w.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

            if child_w.matches > 0: w.children.append(child_w)

    use_thumbnails = pref_use_thumbnails(request)
    message = '<a href="/">Main Page</a> &rarr; <a href="/hilites/show/">HiLites</a> &rarr; ' + str(hilite_obj)
    return render_to_response('hilites/sites.html', locals())


def show_statuses(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Statuses' in view_permissions)): raise Http404

    statuses_obj = Status.objects.all()

    for s in statuses_obj:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            s.matches = s.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
        else:
            s.matches = s.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

    message = '<a href="/">Main Page</a> &rarr; Statuses'
    return render_to_response('statuses/list.html', locals())


def show_status(request, status_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    status_obj = get_object_or_404(Status, pk=status_id)

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs_obj = status_obj.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).order_by('user')[:pref_row_limit(request)]
    else:
        weblogs_obj = status_obj.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).order_by('user')[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; <a href="/status/show/">Statuses</a> &rarr; ' + str(status_obj)
    return render_to_response('statuses/sites.html', locals())


def show_flagged_urls(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Flagged' in view_permissions)): raise Http404

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs_obj = Weblog.objects.filter(user__exclude_from_reports=False).filter(flagged=True)[:pref_row_limit(request)]
    else:
        weblogs_obj = Weblog.objects.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(flagged=True)[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; Flagged &rarr; URLs'
    return render_to_response('flagged/urls.html', locals())


def show_flagged_users(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Flagged' in view_permissions)): raise Http404

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        users_obj = User.objects.filter(exclude_from_reports=False).filter(flagged=True)[:pref_row_limit(request)]
    else:
        users_obj = User.objects.filter(managed_by=duser_obj).filter(exclude_from_reports=False).filter(flagged=True)[:pref_row_limit(request)]

    message = '<a href="/">Main Page</a> &rarr; Flagged &rarr; Users'
    return render_to_response('flagged/users.html', locals())


def flag_user(request, user_id, action):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    if settings.DEMO_MODE == False:
        user = get_object_or_404(User, pk=user_id)

        if not (duser_obj.is_superuser or settings.DISABLE_AUTH):
            if not user.managed_by == duser_obj: return

        user_group, created = UserGroup.objects.get_or_create(name='Flagged Users', description='Flagged Users')

        if (action == 'on'):
            user.flagged = True
            user_group.users.add(user)

        elif (action == 'off'):
            user.flagged = False
            user_group.users.remove(user)

        elif (action == 'toggle'):
            if (user.flagged == True):
                user.flagged = False
                user_group.users.remove(user)
            else:
                user.flagged = True
                user_group.users.add(user)

        user.save()

    return HttpResponse(content=None, status=204)


def flag_url(request, url_id, action):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)

    if settings.DEMO_MODE == False:
        weblog = get_object_or_404(Weblog, pk=url_id)

        if not (duser_obj.is_superuser or settings.DISABLE_AUTH):
            if not weblog.user.managed_by == duser_obj: return

        if (action == 'on'):
            weblog.flagged = True

        elif (action == 'off'):
            weblog.flagged = False

        elif (action == 'toggle'):
            if (weblog.flagged == True):
                weblog.flagged = False
            else:
                weblog.flagged = True

        weblog.save()

    return HttpResponse(content=None, status=204)


def count_weblogs_for_filetype(filetype_obj):
    start_date = pref_start_date(request).strftime("%Y-%m-%d %H:%M:%S")
    end_date   = pref_end_date(request).strftime("%Y-%m-%d %H:%M:%S")

    cursor = connection.cursor()
    sql = \
        "SELECT COUNT(id) FROM log_weblog WHERE mimetype_id=" + str(filetype_obj.id) + \
        " AND log_weblog.log_datetime >= '" + start_date + "' AND log_weblog.log_datetime <= '" + end_date + "'"

    cursor.execute(sql)

    if settings.DEBUG:
        count_tuple = cursor.cursor.fetchall()
    else:
        count_tuple = cursor.fetchall()

    return count_tuple[0][0]


def show_filetypes(request):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Filetypes' in view_permissions)): raise Http404

    all_filetypes_obj = MimeType.objects.all().order_by('type')

    filetypes_obj = []
    for f in all_filetypes_obj:
        if (duser_obj.is_superuser or settings.DISABLE_AUTH):
            f.matches =  f.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()
        else:
            f.matches =  f.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request)).count()

        if f.matches > 0: filetypes_obj.append(f)

    message = '<a href="/">Main Page</a> &rarr; File-types'
    return render_to_response('filetypes/list.html', locals())


def show_filetype(request, mime_id):
    duser_obj = http_login(request)
    view_permissions = get_view_permissions(duser_obj)
    if not ((duser_obj.is_superuser) or ('Filetypes' in view_permissions)): raise Http404

    filetype_obj = get_object_or_404(MimeType, pk=mime_id)

    if (duser_obj.is_superuser or settings.DISABLE_AUTH):
        weblogs_obj = filetype_obj.weblog_set.filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)] 
    else:
        weblogs_obj = filetype_obj.weblog_set.filter(user__managed_by=duser_obj).filter(user__exclude_from_reports=False).filter(log_datetime__gt=pref_start_date(request)).filter(log_datetime__lt=pref_end_date(request))[:pref_row_limit(request)] 

    message = '<a href="/">Main Page</a> &rarr; <a href="/filetypes/show/">Filetypes</a> &rarr; URLs for <em>' + \
        str(filetype_obj) + ' (.' + str(filetype_obj.extension) + ')</em>'

    return render_to_response('filetypes/filetypes.html', locals())












